#
# Find the Windows driver development kit.
#
# The registry will be checked for information about where to find the DDK,
# and some common locations will be examined. If it's not found, you can
# set the -DWINDDK:PATH= argument to cmake to tell it where to look or set the
# WINDDK variable with ccmake .
#

# TODO: detect x86 vs AMD64
# TODO: detect win2k/winxp (probably doesn't matter; just target 2k)
IF(NOT WINDDK_ARCH_NAME)
    SET(WINDDK_ARCH_NAME "i386")
ENDIF(NOT WINDDK_ARCH_NAME)

IF(NOT WINDDK)
    SET(WINDDK "\\WINDDK\\3790.1830" CACHE FORCE "WinDDK install directory")
ENDIF(NOT WINDDK)

FIND_LIBRARY(WINDDK_HID_LIBRARY
    NAMES "hid"
    PATHS
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\WINDDK\\3790.1830;LFNDirectory]/lib/w2k/${WINDDK_ARCH_NAME}"
    "${WINDDK}/lib/w2k/${WINDDK_ARCH_NAME}"
)

FIND_LIBRARY(WINDDK_SETUPAPI_LIBRARY
    NAMES "setupapi"
    PATHS
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\WINDDK\\3790.1830;LFNDirectory]/lib/w2k/${WINDDK_ARCH_NAME}"
    "${WINDDK}/lib/w2k/${WINDDK_ARCH_NAME}"
)

FIND_PATH(WINDDK_INCLUDE_DIR
    NAMES "hidsdi.h"
    PATHS
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\WINDDK\\3790.1830;LFNDirectory]/inc/w2k"
    "${WINDDK}/inc/w2k"
)

SET(WINDDK_LIBRARY "${WINDDK_SETUPAPI_LIBRARY}" "${WINDDK_HID_LIBRARY}")
INCLUDE(LprofFindPackageHandleStandardArgs)
FIND_PACKAGE_HANDLE_STANDARD_ARGS(WINDDK DEFAULT_MSG WINDDK_LIBRARY WINDDK_INCLUDE_DIR)

IF(WINDDK_FOUND)
    SET(WINDDK_LIBRARIES ${WINDDK_LIBRARY})
ENDIF(WINDDK_FOUND)
