#
# Must be included after qt4 has been detected
# This code sets up a custom command to output
# compiled .ts files from the .qm files.
#

GET_FILENAME_COMPONENT(QTTRANS_QTDIR "${QT_MOC_EXECUTABLE}" PATH)
FIND_PROGRAM(QTTRANS_LRELEASE_EXECUTABLE
    NAMES lrelease
    PATHS "${QTTRANS_QTDIR}"
    DOC "Path to qt lrelease executable"
)

#
# compile translations using lrelease.
# Usage:
#
# QTTRANS_COMPILE_TRANSLATIONS(
#    output_variable
#    qm_output_directory
#    input_ts_list
# )
#
# "output_variable" should be the name of a variable that will
# be set to a list of the names of all the generated qm files.
# Add the qm file list to the sources of the appropriate build
# target and they'll be built when the target gets built.
#
# qm_output_directory is the directory to save the files in.
# It should usually be ${CMAKE_CURRENT_BINARY_DIRECTORY}
#
# input_ts_list is a list of one or more .ts file paths
# to compile. You can generate this with a FILE glob.
#
# Note that this macro doesn't actually compile the files, it
# just defines some build rules permitting them to be compiled.
# It will have no effect unless the output file list is added
# to a target's sources.
#
MACRO(QTTRANS_COMPILE_TRANSLATIONS output_var output_dir)
    FILE(MAKE_DIRECTORY "${output_dir}")
    SET(qm_files)
    FOREACH(ts_file ${ARGN})
        GET_FILENAME_COMPONENT(ts_basename "${ts_file}" NAME_WE)
        SET(qm_file "${output_dir}/${ts_basename}.qm")
        ADD_CUSTOM_COMMAND(
            OUTPUT "${qm_file}"
            COMMAND "${QTTRANS_LRELEASE_EXECUTABLE}" "${ts_file}" "-qm" "${qm_file}"
            DEPENDS "${ts_file}"
            COMMENT "Compile translation"
        )
        SET(qm_files ${qm_files} "${qm_file}")
    ENDFOREACH(ts_file)
    SET("${output_var}" ${qm_files})
ENDMACRO(QTTRANS_COMPILE_TRANSLATIONS)
